
local utils = require('utils')

-- Global list of entities to replenish energy on every tick.
-- It is manually re-populated on game load and maintained during the game.
-- FIXME: For some reason, setting .energy of these entities to a ridiculously
-- large value didn't work, so we'd have to deal with on_tick for now;
-- otherwise we could just supply these entities with a few billion years worth
-- of energy upon construction.
infinite_energy_entities = {}
infinite_energy_entities_populated = false

script.on_event(defines.events.on_player_created, function(event)
  local player = game.players[event.player_index]
  player.get_quickbar().clear()
end)

local function hack_replace_lava_furnace(entity)
  local surface = entity.surface
  local position = entity.position
  local force = entity.force
  local direction = entity.direction

  entity.destroy()

  local replacement = surface.create_entity({
     name = 'hack-lava-furnace-' .. utils.direction_names[direction],
     position = { x = position.x - 0, y = position.y - 0 },
     force = force
   })

  replacement.energy = 1
  utils.append(infinite_energy_entities, replacement)
end

script.on_event(defines.events.on_built_entity, function(event)
  local entity = event.created_entity
  if entity.name == 'hack-lava-furnace-as-pump' then
    hack_replace_lava_furnace(entity)
  end
end)

script.on_event(defines.events.on_robot_built_entity, function(event)
  local entity = event.created_entity
  if entity.name == 'hack-lava-furnace-as-pump' then
    hack_replace_lava_furnace(entity)
  end
end)

script.on_event(defines.events.on_tick, function(event)
  if not infinite_energy_entities_populated then
    for k, surface in pairs(game.surfaces) do
      for _, direction_name in pairs(utils.direction_names) do
        entity_name = 'hack-lava-furnace-' .. direction_name
        entities = surface.find_entities_filtered({name=entity_name})
        for _, entity in ipairs(entities) do
          utils.append(infinite_energy_entities, entity)
        end
      end
    end
    infinite_energy_entities_populated = true
  end
  local valid_infinite_energy_entities = {}
  for _, v in pairs(infinite_energy_entities) do
    if v.valid then
      v.energy = 1
      utils.append(valid_infinite_energy_entities, v)
    end
  end
  infinite_energy_entities = valid_infinite_energy_entities
end)
