
local utils = require('utils')
local autoplace_utils = require("autoplace_utils")

function main()
  replace_terrains()
  remove_default_decoratives()
  remove_default_trees()
  remove_misc_default_stuff()
end

--------------------------------------------------------------------------------
-- A little more specific stuff.

function replace_terrains()
  for terrain_name, params in pairs({
    ['grass']        = { map_color = { r = 0.20, g = 0.20, b = 0.20 }},
    ['grass-medium'] = { map_color = { r = 0.18, g = 0.18, b = 0.18 }},
    ['grass-dry']    = { map_color = { r = 0.16, g = 0.16, b = 0.16 }},
    ['dirt']         = { map_color = { r = 0.14, g = 0.14, b = 0.14 }},
    ['sand']         = { map_color = { r = 0.26, g = 0.26, b = 0.26 }},
    ['sand-dark']    = { map_color = { r = 0.22, g = 0.22, b = 0.22 }},
    ['water']        = { map_color = { r = 0.55, g = 0.25, b = 0.15 }},
    ['deepwater']    = { map_color = { r = 1.00, g = 0.40, b = 0.20 }}
  }) do
    replace_terrain(terrain_name, params.map_color)
  end
end

function remove_default_decoratives()
  -- Remove all decorative (grass, bush) stats, apart from the new lava cracks.
  for k, v in pairs(data.raw['decorative']) do
    if k ~= 'lava-crack' then
      data.raw['decorative'][k] = nil
    end
  end
end

function remove_default_trees()
  -- Remove all tree stats, apart from the new crystal stats.
  for k, v in pairs(data.raw['tree']) do
    if not utils.startswith(k, 'crystal-') then
      data.raw['tree'][k] = nil
    end
  end
end

function remove_misc_default_stuff()
  -- I didn't come up with a good replacement for these.
  data.raw['simple-entity']['stone-rock'] = nil
  -- No fish in lava.
  data.raw['fish']['fish'] = nil
end

--------------------------------------------------------------------------------
-- Terrain textures.

-- In our poorman's tileset, all tiles have the same number of variants,
-- and all variants have same probability.
-- This function loads the tiles of the given terrain (identified by name)
-- from the mod and replaces the respective base game tiles.
-- Numbers of variants are modified accordingly.
function replace_terrain(terrain_name, map_color)
  local function get_tex_path(suffix)
    return string.format('__%s__/graphics/terrain/%s/%s%s.png',
                         utils.mod_name, terrain_name, terrain_name, suffix)
  end

  local function get_uniform_probabilities(n)
    local list = {}
    for i = 1, n do
      list[i] = 1 / n
    end
    return list
  end

  data.raw["tile"][terrain_name].variants = {
    inner_corner = {
      picture = get_tex_path('-inner-corner'),
      count = 8
    },
    outer_corner = {
      picture = get_tex_path('-outer-corner'),
      count = 8
    },
    side = {
      picture = get_tex_path('-side'),
      count = 8
    },
    main = {
      {
        picture = get_tex_path('1'),
        count = 16,
        line_length = 8,
        size = 1,
        weights = get_uniform_probabilities(16)
      },
      {
        picture = get_tex_path('2'),
        count = 32,
        line_length = 8,
        size = 2,
        probability = 10,
        weights = get_uniform_probabilities(32)
      },
      {
        picture = get_tex_path('4'),
        count = 16,
        line_length = 4,
        size = 4,
        probability = 100,
        weights = get_uniform_probabilities(16)
      },
    },
  }
  data.raw["tile"][terrain_name].map_color = map_color
end

--------------------------------------------------------------------------------
-- Finally do something.

main()
